#ifndef TRACKERGREP_GREPPER_H
#define TRACKERGREP_GREPPER_H

// ***************************************************************************
// System Includes
// ***************************************************************************

#include <StorageDefs.h>
#include <OS.h>
#include <Entry.h>
#include <Directory.h>
#include <List.h>
#include <Message.h>
#include <Looper.h>
#include <NodeInfo.h>
#include <Path.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

// ***************************************************************************
// Project Includes
// ***************************************************************************

#include "StyleGuideDefs.h"          // Standard type and constant definitions

// ***************************************************************************
// External Name Declarations
// ***************************************************************************

// ***************************************************************************
// External Variable Declarations
// ***************************************************************************

// ***************************************************************************
// Constant Definitions
// ***************************************************************************

/** The messages that the grepper object sends to its target. */
enum {
  /** Sends the name of the next file that will be grepped. */
  MSG_REPORT_FILE_NAME = 2000,
  
  /**
   * Sends the output of a grep operation. This message will only be sent
   * a certain file's contents match the search pattern. 
   */
  MSG_REPORT_RESULT,
  
  /** Indicates that the search has finished. */
  MSG_SEARCH_FINISHED,
};

/** The maximum length of the "text" string inside a report message. */
const tInt32 MAX_LEN_REPORT_TEXT = B_PATH_NAME_LENGTH;
  
// ***************************************************************************
// Type Definitions
// ***************************************************************************

// ***************************************************************************
// Class Definition
// ***************************************************************************

/**
 * The tGrepper Class
 *
 * A tGrepper object is responsible for executing grep with a certain search
 * pattern on a certain number of files. If the user selected one or more 
 * files, then only these files will be processed. If nothing was selected, 
 * then all the files from the current working directory will be grepped. 
 * Recursive crawling of subdirectories is optional. While grep is running,
 * the grepper object will send messages with the results of the operation 
 * to a certain target.
 *
 * Typical usage is as follows:
 *
 * 1) Create the grepper object.
 * 2) Connect it to a target BLooper (for instance a BWindow).
 * 3) Set the search pattern.
 * 4) Determine whether subdirectories should be recursed or not.
 * 5) Call Start() to begin.
 * 6) Do something else while the grepper thread does its job. 
 * 7) The grepper thread will terminate automatically when all the files are
 *    processed, but you may call Stop() to abort it.
 * 
 * @Version 1.1.1
 * @Date    Friday, 31 July 1998
 * @Since   Tracker Grep Version 2.0.0
 */
class tGrepper {
public:
  // Constant Definitions ====================================================

  // Type Definitions ========================================================

  // Attribute Definitions ===================================================

  // Member Function Definitions =============================================

  // Lifecycle ---------------------------------------------------------------

  /** 
   * Constructor. 
   *
   * @Params aDirRef The entry_ref of the current directory. 
   *         This may never be NULL !
   * @Params apMessage A standard B_REFS_RECEIVED message with the "refs"
   *         array containing the entry_refs of the files selected by the 
   *         user. This may never be NULL !
   */
  tGrepper(entry_ref aDirRef, BMessage* apMessage);

  /** Automatically generated copy constructor. */
  // tGrepper(const tGrepper& arOld);

  /** Destructor. */
  ~tGrepper();

  // Operators ---------------------------------------------------------------

  /** Automatically generated assignment operator. */
  // tGrepper& operator=(tGrepper& arRight);

  // Operations --------------------------------------------------------------
  
  /** 
   * Starts a new grepper thread. 
   *
   * The new thread isn't started if the search pattern is empty. Any running
   * grepper threads are not stopped by this function. You will have to call
   * Stop() yourself first.
   *
   * @Returns B_OK if everything went fine, or an error code otherwise. 
   *
   * @SeeAlso resume_thread() (The Be Book, Kernel Kit, Threads) for all
   *          possible return values.
   */
  status_t Start();

  /**
   * Stops (and terminates) the grepper thread. 
   *
   * Note that the thread will not send a MSG_SEARCH_FINISHED message in this
   * case.
   */
  tVoid Stop(); 
   
  // Access ------------------------------------------------------------------

  /** Sets the target that the we will send our messages to. */
  tVoid SetTarget(BLooper* apTarget);

  /** Sets the search pattern. */
  tVoid SetPattern(const tChar* apPattern);

  /**
   * Determines whether subdirectories should be crawled or not. 
   * Defaults to TRUE.
   */
  tVoid SetRecurseDirs(tBool aRecurse);

  /**
   * Determines whether symbolic links should be traversed or not. 
   * Defaults to FALSE.
   */
  tVoid SetRecurseLinks(tBool aRecurse);
       
  // Inquiry -----------------------------------------------------------------

protected:
  // Constant Definitions ====================================================

  // Type Definitions ========================================================

  // Attribute Definitions ===================================================

  // Member Function Definitions =============================================

private:
  // Constant Definitions ====================================================

  // Type Definitions ========================================================

  // Attribute Definitions ===================================================

  /** List that contains pointers to BDirectory objects. */
  BList* mpDirs;  

  /** The current working directory at the time Tracker Grep was invoked. */
  BDirectory* mpInitialDir;

  /** The current working directory. (Used for subdirectory crawling.) */
  BDirectory* mpCurrentDir;

  /** A B_REFS_RECEIVED message containing the files selected by the user. */
  BMessage* mpMessage;

  /** Number of the current entry_ref in the B_REFS_RECEIVED message. */
  tInt32 mCurrentRef;
    
  /** Pointer to the target BLooper. */
  BLooper* mpTarget;

//  /** Pointer to the search pattern. */
//  tChar* mpPattern;
  /** 
   * The search pattern. This will break if the search pattern is larger
   * than 1024 characters, but that's very unlikely.
   */
  tChar mPattern[1024];
      
  /** TRUE if we need to look into subdirectories, FALSE if we don't. */
  tBool mRecurseDirs;

  /** TRUE if we need to look into symbolic links, FALSE if we don't. */
  tBool mRecurseLinks;
  
  /** Thread ID of the current grepper thread. */
  thread_id mGrepperThreadId;
  
  /** TRUE if the grepper thread must quit, FALSE if it may continue. */ 
  tBool mThreadMustQuit;

  // Member Function Definitions =============================================

  /** Stub that spawns the real grepper thread. */
  static int32 SpawnThread(void* apArg); 

  /** 
   * The thread function that does the actual grepping. 
   *
   * @Returns Always 0.
   */
  tInt32 GrepperThread(void); 

  /**
   * Returns the full path name of the next file that needs to be grepped. 
   *
   * @Param apBuffer GetNextName() copies the path name into this buffer. It 
   *        must be allocated before it's passed in. The buffer must be large
   *        enough to accommodate the name; B_PATH_NAME_LENGTH is a 100% safe
   *        bet.
   *
   * @Returns B_OK if a name was found, B_ERROR if there are no more names.
   */
  status_t GetNextName(tChar* apBuffer);

  /**
   * Changes all " characters inside a certain string into \" characters.
   *
   * The BeOS allows file names to contain all characters, except for /.
   * Unfortunately, the shell is more restrictive and can't handle spaces 
   * inside file names. Therefore, Tracker Grep surrounds all file names with
   * double quotes (") to prevent the shell from misinterpreting spaces, 
   * brackets and the like. But this causes problems for file names that
   * already contain double quotes themselves. The solution is to prefix (or
   * "escape") such characters with a back slash.
   *
   * @Param apBuffer The string that we will be modifying. The buffer must be
   *        large enough to accomodate the changes. B_PATH_NAME_LENGTH is
   *        usually safe, since the name usually contains only a few "s.
   */
  tVoid AdjustFunnyCharacters(tChar* apBuffer);
};

// ***************************************************************************
// Global Variable Declarations
// ***************************************************************************

// ***************************************************************************
// Inline Function Definitions
// ***************************************************************************

// ===========================================================================
// SetTarget
// ===========================================================================

inline tVoid
tGrepper::SetTarget(BLooper* apTarget)
{
  mpTarget = apTarget;
}

// ===========================================================================
// SetPattern
// ===========================================================================

inline tVoid
tGrepper::SetPattern(const tChar* apPattern)
{
  // Changed by Matthijs on 23 Jan 1999: we must copy the entire string, and
  // not just the pointer to it. The reason for this is that we will call this
  // function and pass it the pointer of the BTextControl's text. But during
  // the grepping, we will be changing the text of this control, which would
  // cause the pattern to change as well!
  
  // mpPattern = (tChar*) apPattern;
  strcpy(mPattern,apPattern);
}

// ===========================================================================
// SetRecurseDirs
// ===========================================================================

inline tVoid 
tGrepper::SetRecurseDirs(tBool aRecurse)
{
  mRecurseDirs = aRecurse;
}

// ===========================================================================
// SetRecurseLinks
// ===========================================================================

inline tVoid 
tGrepper::SetRecurseLinks(tBool aRecurse)
{
  mRecurseLinks = aRecurse;
}

// ***************************************************************************
// Helper Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Function Declarations
// ***************************************************************************

#endif // TRACKERGREP_GREPPER_H
